from vcCommand import *
from vcHelpers.Application import *
import math, vcMatrix, random

app = getApplication()
selection = app.SelectionManager

def OnStart():
  # Check if only 1 component is selected
  if len(selection.getSelection(VC_SELECTION_COMPONENT)) == 1:
    # Selected component
    target = selection.getSelection(VC_SELECTION_COMPONENT)[0]
    # Check if selected component has property for activate physics
    targetActivatePhysics = target.getProperty("ActivatePhysics")
    if not targetActivatePhysics:
      targetActivatePhysics = target.createProperty(VC_BOOLEAN, "ActivatePhysics")
    targetActivatePhysics.Value = False
    
    # Check if selected component has python script for activate physics
    targetActivateScript = target.findBehaviour("ActivateScript")
    if not targetActivateScript:
      targetActivatePhysics = target.createBehaviour(VC_PYTHONSCRIPT, "ActivateScript")    
    targetActivateScript.Script = '''
from vcScript import *

def changePhysicsType(type):
  for child in app.Components:
    if ("%s__" % comp.Name) in child.Name:
      physics = child.findBehaviour("PhysicsEntity")
      physics.PhysicsType = type

def OnActivatePhysics(arg):
  if activatePhysics.Value:
    changePhysicsType(VC_PHYSICS_INPHYSICS)
  else:
    changePhysicsType(VC_PHYSICS_KINEMATIC)
  
app = getApplication()
comp = getComponent()
activatePhysics = comp.getProperty("ActivatePhysics")
activatePhysics.OnChanged = OnActivatePhysics
'''    
    
    # Delete previous created pieces
    for part in app.Components:
      if ("%s__" % target.Name) in part.Name:
        part.delete()
    
    # Get selected component bound space
    minCorner = target.BoundCenter - target.BoundDiagonal * 1.05
    maxCorner = target.BoundCenter + target.BoundDiagonal * 1.05
    diff = maxCorner - minCorner
    
    # Fixed parameter for now
    size = 1000.0   
    
    # Calculate how many pieces for x, y, z
    countX = int(math.ceil(diff.X / size))
    countY = int(math.ceil(diff.Y / size))
    countZ = int(math.ceil(diff.Z / size))    
        
    # Fixed parameter for now
    geometry = target.findFeature("Geometry")
    if geometry:    
      
      for zIndex in range(countZ):
        for yIndex in range(countY):
          for xIndex in range(countX):
            # Divide box into 2 wedge, make it more natural
            ratio = random.uniform(0.1, 0.9)
            
            # Down wedge
            partDown = app.createComponent()
            partDown.Name = "%s__%i_%i_%i_Down" % (target.Name, xIndex, yIndex, zIndex)
            partDown.PositionMatrix = target.PositionMatrix
            
            # Create physics entity
            physicsDown = partDown.createBehaviour(VC_PHYSICSENTITY, "PhysicsEntity")
            physicsDown.PhysicsType = VC_PHYSICS_KINEMATIC
            
            # Create boolean feature
            booleanDown = partDown.RootFeature.createFeature(VC_BOOLEAN, "Boolean")
            booleanDown.Operation = VC_BOOLEAN_INTERSECTION
            
            # Create wedge feature
            wedgeDown = booleanDown.createFeature(VC_WEDGE, "Down")
            wedgeDown.BiggerLength = "%f {mm}" % size
            wedgeDown.SmallerLength = "%f {mm}" % (size * ratio)
            wedgeDown.BiggerHeight = "%f {mm}" % size
            wedgeDown.SmallerHeight = "%f {mm}" % (size * ratio)
            wedgeDown.Width = "%f {mm}" % size
            
            # Wedge matrix
            matrixDown = vcMatrix.new()
            matrixDown.translateRel(minCorner.X, minCorner.Y, minCorner.Z)
            matrixDown.translateRel(xIndex * size, yIndex * size, zIndex * size)
            wedgeDown.PositionMatrix = matrixDown
            
            # Clone geometry
            geometryDown = booleanDown.createFeature(VC_GEOMETRY, "Geometry")            
            geometry.Geometry.cloneGeometrySets(geometryDown.Geometry)
            
            # After intersection, collapse
            booleanDown.collapse()
            booleanDown = partDown.findFeature("Boolean")            
            booleanDown.PhysicsCollider = VC_PHYSICSCOLLIDER_PRECISE
            # Check if not intersection
            if booleanDown.Geometry.BoundDiagonal.X < 1e-3 and booleanDown.Geometry.BoundDiagonal.Y < 1e-3 and booleanDown.Geometry.BoundDiagonal.Z < 1e-3:
              partDown.delete()
            
            # Up wedge
            partUp = app.createComponent()
            partUp.Name = "%s__%i_%i_%i_Up" % (target.Name, xIndex, yIndex, zIndex)
            partUp.PositionMatrix = target.PositionMatrix
            
            # Create physics entity
            physicsUp = partUp.createBehaviour(VC_PHYSICSENTITY, "PhysicsEntity")
            physicsUp.PhysicsType = VC_PHYSICS_KINEMATIC
            
             # Create boolean feature
            booleanUp = partUp.RootFeature.createFeature(VC_BOOLEAN, "Boolean")
            booleanUp.Operation = VC_BOOLEAN_INTERSECTION
            
            # Create wedge feature
            wedgeUp = booleanUp.createFeature(VC_WEDGE, "Up")                      
            wedgeUp.BiggerLength = "%f {mm}" % (size * (1.0 - ratio))
            wedgeUp.SmallerLength = "0.0 {mm}"
            wedgeUp.BiggerHeight = "%f {mm}" % (size * (1.0 - ratio))
            wedgeUp.SmallerHeight = "0.0 {mm}"
            wedgeUp.Width = "%f {mm}" % size
            
            # Wedge matrix, this one needs to turn down
            matrixUp = vcMatrix.new()
            matrixUp.translateRel(minCorner.X, minCorner.Y, minCorner.Z)
            matrixUp.translateRel((xIndex + 1) * size, (yIndex + 1) * size, (zIndex + 1) * size)
            matrixUp.setWPR(180.0, 90.0, 0.0)
            wedgeUp.PositionMatrix = matrixUp
            
            # Clone geometry
            geometryUp = booleanUp.createFeature(VC_GEOMETRY, "Geometry")            
            geometry.Geometry.cloneGeometrySets(geometryUp.Geometry)
            
             # After intersection, collapse
            booleanUp.collapse()
            booleanUp = partUp.findFeature("Boolean")    
            booleanUp.PhysicsCollider = VC_PHYSICSCOLLIDER_PRECISE            
            # Check if not intersection
            if booleanUp.Geometry.BoundDiagonal.X < 1e-3 and booleanUp.Geometry.BoundDiagonal.Y < 1e-3 and booleanUp.Geometry.BoundDiagonal.Z < 1e-3:
              partUp.delete()
           